#!/usr/bin/perl
#-----------------------------------------------------------------------
# captcha.cgi - generate a captcha image
#
# Usage:  captcha.cgi?XXXXXXX...
#
# Query string is an encrypted version of the string that should be 
# encoded into the captcha image.
#
# WARNING: the default image size will only accept 5 characters.
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Crypt;

if ($ENV{HTTP_USER_AGENT} =~ /msnbot|Googlebot|Yahoo/i) {
    # looks like a robot - don't even bother
    print "content-type:text/plain\n\n";
    print "Silly robot.";
    exit;
}

&exsite_init;

# basic captcha configuration

my $captcha_command = "convert -font [[font]] [[size]] [[text]] -background white -fill '[[color]]' [[distort]] $config{server}{HTMLroot}$config{server}{HTMLpath}/_ExSite/images/captcha.png -";

my $x = $config{captcha}{start_x};
my $y = $config{captcha}{start_y};

my %param = (
	     font=>$config{captcha}{font},
	     size=>"-pointsize $config{captcha}{pointsize}",
	     color=>$config{captcha}{color},
	     );

# get our captcha text

my $crypttext = $ENV{QUERY_STRING};
my $c = new ExSite::Crypt;
my $plaintext = $c->decrypt($crypttext);

# generate commands to write the text on the image

my @text = split //,$plaintext;
foreach my $t (@text) {
    my $y2 = $y + int(rand(10))-5;
    $param{text} .= "-draw \"text $x,$y2 '$t'\" ";
    $x += $config{captcha}{charsize};
}

# distort?

if ($config{captcha}{distort}) {
    $param{distort} = "-implode $config{captcha}{distort}";
}

# generate the full image processing command

my $cmd = &substitute($captcha_command,\%param);

# create and output the captcha image

print "content-type:image/png\n\n";
print `$cmd`;

