#!/usr/bin/perl
#!/usr/bin/speedy
#-----------------------------------------------------------------------
# ctrl-panel.cgi - generate a control panel for a web app
#
# Usage:  /cgi-bin/ctrl-panel.cgi/WEBAPP?options
#
# WEBAPP is the name of an application installed in the Modules area.
#
# All options are assumed to be directed at the Webapp, except:
# 
# _bare=1    Do not show the usual ExSite webtop wrapper.
#
#-----------------------------------------------------------------------
#
#   Copyright 2001-2007 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::PageBuf;
use ExSite::Form;  # use Form if inputting or processing forms
use ExSite::Module qw(get_module icon);
use ExSite::Misc;

use vars qw($db $page %query);

&exsite_init;
$db = new ExSite::Form;
%query = &DecodeQuery;

# enable page buffering for better redirect control

my @path = split /\//, $ENV{PATH_INFO};
shift @path;
my $module = shift @path;

my $BARE = $query{_bare};

my %page_opt = (title=>"$module - $config{site}{name}");
#if ($BARE) {
    $page_opt{header} = $page_opt{footer} = 0;
#}
#else {
#    $page_opt{headtype} = 1;
#}
$page = new ExSite::PageBuf(%page_opt);

my ($icon,$name,$info,$out);

if ($db->authorize > 1) {
    my $app = &get_module($module);
    if ($app) {
	$icon = $app->icon;
	$name = $app->ioctl('ModuleName') || $module;
	$info = $app->ioctl('ModuleInfo');
	my $panel = $app->ioctl('ControlPanel');
	if (ref $panel eq "CODE") {
	    $out .= ( $app->$panel || $@ );
	}
	else {
	    $out .= "<h1>Error executing $module control panel</h1>\n";
	}
    }
    else {
	if ($@) {
	    $out .= "<h1>Failed to start module $module</h1>\n<p>$@</p>\n";
	}
	else {
	    $out .= "<h1>$module: Permission denied (no license)</h1>\n";
	}
    }
}
else {
    $out .= "<h1>Permission denied.</h1>
<p>You do not have permission to access this control panel.</p>\n";
    $out .= $share{DB}->login_form(undef,$config{server}{server}.&relink());
}

my $help;
if (-e "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/$module/help.html") {
    $help = "<a href=\"javascript:popup_medium('$config{server}{CGIpath}/$config{prog}{help}/_Modules/$module/help.html')\">help</a>";
}

if ($BARE) {
    $page->printTo(2,"<html>
<head>
<title>$module</title>
<link REL=\"stylesheet\" TYPE=\"text/css\" HREF=\"$config{server}{HTMLpath}/_ExSite/css/ExSite.css\">
</head>
<body class=\"ctrlpanel\">
<div style=\"float:right; font-size:9pt;\">
<a href=\"javascript:window.close()\">close</a>
</div>\n");
    $page->printTo(3,"</body></html>");

    $page->print($out);
}
else {
    if (! $page->{section}[2]) {
	$page->printTo(2,"<html>
<head>
<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">
<title>$module</title>
<link REL=\"stylesheet\" TYPE=\"text/css\" HREF=\"$config{server}{HTMLpath}/_ExSite/css/ExSite.css\">
<script type='text/javascript' src='$config{server}{HTMLpath}/_ExSite/js/misc.js'></script>
</head>
<body MARGINHEIGHT=\"0\" MARGINWIDTH=\"0\" TOPMARGIN=\"0\" LEFTMARGIN=\"0\" RIGHTMARGIN=\"0\" class=\"ctrlpanel\">
<table width=\"100%\" cellspacing=0 cellpadding=0 border=0 class=\"titlebar\">
<tr><td rowspan=2 class=\"titlebar_noborder\"><img src=\"$icon\" align=\"left\"></td>
<td class=titlebar_noborder><span class=\"title\">$name</span><br>\n$info</td>
</tr>
<tr><td valign=bottom align=right class=titlebar_noborder>
$help &nbsp;&nbsp;
<a href=\"javascript:history.back()\">back</a> &nbsp;&nbsp;
<a href=\"javascript:window.location.reload()\">refresh</a> &nbsp;&nbsp;
<a href=\"javascript:window.close()\">close</a> 
</td></tr></table>
<div class=\"ctrlpanel\">
\n");
    }
    if (! $page->{section}[3]) {
	$page->printTo(3,"</div></body></html>");
    }
    $page->print($out);
}
$page->show;

&exsite_close();

### END

